#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet_x/enterprisemodules/ora_install/get_homes'

Puppet::Type.type(:ora_opatch).provide(:base) do
  include EasyType::Provider
  extend ::PuppetX::EnterpriseModules::OraInstall::GetHomes

  def os_user
    @os_user ||= self.class.user_for_file(oracle_product_home_dir)
  end

  desc 'Base provider for Oracle patches'

  mk_resource_methods

  confine :true => false # This is NEVER a valid provider. It is just used as a base class

  def self.prefetch(resources)
    orainst_dir = check_single_orainst_dir(resources)
    all_patches = oracle_homes(resources).collect { |h| patches_in_home(h, orainst_dir).collect { |p| p[:name] } }.flatten
    resources.keys.each do |patch_name|
      if all_patches.include?(patch_name)
        resources[patch_name].provider = new(:name => name, :ensure => :present)
      end
    end
  end

  def opatch(command, options)
    options = { :failonfail => true, :uid => os_user, :combine => true }.merge(options)
    Dir.chdir(oracle_product_home_dir) do
      full_command = "export ORACLE_HOME=#{oracle_product_home_dir}; #{oracle_product_home_dir}/OPatch/opatch #{command}"
      output = Puppet::Util::Execution.execute(full_command, options)
      Puppet.info output
      raise 'Opatch contained an error' unless output =~ /OPatch completed|OPatch succeeded|opatch auto succeeded|opatchauto succeeded/
      output
    end
  end

  def opatchauto(command, options)
    options = { :failonfail => true, :combine => true }.merge(options)
    Dir.chdir(oracle_product_home_dir) do
      full_command = "export ORACLE_HOME=#{oracle_product_home_dir}; #{oracle_product_home_dir}/OPatch/opatchauto #{command}"
      output = Puppet::Util::Execution.execute(full_command, options)
      Puppet.info output
      output
    end
  end

  [:patch_id, :oracle_product_home_dir, :orainst_dir, :source, :tmp_dir].each do |prop|
    define_method(prop) do
      resource[prop]
    end
  end

  private

  def self.patches_in_home(oracle_product_home_dir, orainst_dir)
    os_user = user_for_file(oracle_product_home_dir)
    Dir.chdir(oracle_product_home_dir) do
      full_command = "#{oracle_product_home_dir}/OPatch/opatch lspatches -oh #{oracle_product_home_dir} -invPtrLoc #{orainst_dir}/oraInst.loc"
      raw_list = Puppet::Util::Execution.execute(full_command, :failonfail => true, :uid => os_user)
      Puppet.info "\n#{raw_list}"
      raw_list.scan(/^(\d+);(.*)$/).collect { |e| { :name => "#{oracle_product_home_dir}:#{e[0]}", :comment => e[1] } }
    end
  end

  def self.check_single_orainst_dir(resources)
    orainst_dir = resources.map { |k, v| v.orainst_dir }.uniq
    raise "ora_opatch doesn't support multiple orainst_dir" if orainst_dir.size > 1
    orainst_dir.first
  end
end
