module PuppetX
  module EnterpriseModules
    # Docs
    module OraInstall
      # Docs
      module GetHomes

        def default_database_sid
          Hash[*oratab_content.scan(/^(?!#)(.*)\:(.*)\:.*$/).flatten].keys.first
        end

        def oracle_home_for(sid)
          Hash[*oratab_content.scan(/^(?!#)(.*)\:(.*)\:.*$/).flatten].select { |k, _v| k == sid }.values.first
        end

        def user_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          a_file = first_non_root_file(home)
          uid = File.stat("#{home}/#{a_file}").uid
          Etc.getpwuid(uid).name
        end

        def first_non_root_file(dir)
          Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
        end

        def oracle_homes(resources = nil)
          if resources
            resources.map { |k, v| v.oracle_product_home_dir }.uniq
          else
            oratab_content.scan(/^(?!#).*\:(.*)\:.*$/).flatten.uniq
          end
        end

        def installed_patches
          oracle_homes.collect do |oracle_home|
            patches_in_home(oracle_home, ora_invdir)
          end.flatten
        end

        def oratab_content
          File.read(oratab_file)
        end

        def oratab_file
          "#{ora_invdir}/oratab"
        end

        def ora_invdir
          os = Facter.value(:kernel)
          case os
          when 'Linux' then
            '/etc'
          when 'SunOS' then
            '/var/opt/oracle'
          else
            '/etc'
          end
        end
      end
    end
  end
end
