#
# See the file "LICENSE" for the full license governing this code.
#
# == Define:  ora_install::autostartdatabase
#
#  autostart of the database for linux and SunOS
#
define ora_install::autostartdatabase(
  Stdlib::Absolutepath                   $oracle_home  = undef,
  String[1]                              $db_name      = undef,
  String[1]                              $user         = 'oracle',
  String[1]                              $group        = 'dba',
  String[1]                              $service_name = 'dbora',
  Enum['database','asm','grid']          $db_type      = 'database',
  Enum['OPEN','MOUNT','READ ONLY']       $start_option = 'OPEN',
  Enum['AUTOMATIC','MANUAL','NORESTART'] $db_policy    = 'AUTOMATIC',
  Optional[String[1]]                    $db_domain    = undef,
  Variant[Boolean,Enum['on_failure']]    $logoutput    = lookup({name => 'logoutput', default_value => 'on_failure'}),
){

  ora_install { $title: ensure => 'present'}
  $exec_path    = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  if $db_type == 'grid' {

    if $db_domain {
      $domain_clause = "-m ${db_domain}"
    } else {
      $domain_clause = ''
    }
    exec { "adding database ${db_name}:${oracle_home} as has service":
      command     => "${oracle_home}/bin/srvctl add database -d ${db_name} -o ${oracle_home} -s ${start_option} -n ${db_name} -y ${db_policy} ${domain_clause}",
      onlyif      => "${oracle_home}/bin/srvctl status database -d ${db_name} | grep -q 'Resource ora.*.db does not exist'",
      environment => ["ORACLE_SID=${db_name}","ORACLE_HOME=${oracle_home}",'ORAENV_ASK=NO'],
      cwd         => $oracle_home,
      user        => $user,
      group       => $group,
      path        => $exec_path,
      logoutput   => $logoutput,
    }
    ~> exec { "updating status of database ${db_name}:${oracle_home}":
        command     => "${oracle_home}/bin/srvctl start database -d ${db_name}",
        onlyif      => "${oracle_home}/bin/srvctl status database -d ${db_name} | grep -q 'Database is not running.'",
        environment => ["ORACLE_SID=${db_name}","ORACLE_HOME=${oracle_home}",'ORAENV_ASK=NO'],
        cwd         => $oracle_home,
        user        => $user,
        group       => $group,
        path        => $exec_path,
        refreshonly => true,
        logoutput   => $logoutput,
      }
  }
  else {

    ora_install::prepareautostart_for { $service_name:
      oracle_home => $oracle_home,
      user        => $user,
    }

    case $::kernel {
      'Linux': {
        $ora_tab = '/etc/oratab'
        $dbora_location = '/etc/init.d'
        $sed_command = "sed -i -e's/:N/:Y/g' ${ora_tab}"
      }
      'SunOS': {
        $ora_tab = '/var/opt/oracle/oratab'
        $dbora_location = '/etc'
        $sed_command = "sed -e's/:N/:Y/g' ${ora_tab} > /tmp/oratab.tmp && mv /tmp/oratab.tmp ${ora_tab}"
      }
      default: {
        fail('Unrecognized operating system, please use it on a Linux or SunOS host')
      }
    }

    exec { "set dbora ${db_name}:${oracle_home}":
      command   => $sed_command,
      unless    => "/bin/grep '^${db_name}:${oracle_home}:Y' ${ora_tab}",
      require   => File["${dbora_location}/dbora"],
      path      => $exec_path,
      logoutput => $logoutput,
    }
  }

}
