#
# See the file "LICENSE" for the full license governing this code.
#
#
#
define ora_install::database_pluggable(
  Enum[present,absent]                          $ensure                   = present,
  Enum['12.1','12.2']                           $version                  = '12.1',
  Stdlib::Absolutepath                          $oracle_home_dir          = undef,
  String[1]                                     $user                     = 'oracle',
  String[1]                                     $group                    = 'dba',
  String[1]                                     $source_db                = undef,
  String[1]                                     $pdb_name                 = undef,
  String[1]                                     $pdb_datafile_destination = undef,
  String[1]                                     $pdb_admin_username       = 'pdb_adm',
  String[1]                                     $pdb_admin_password       = undef,
  Boolean                                       $create_user_tablespace   = true,
  Optional[Variant[Boolean,Enum['on_failure']]] $log_output               = undef,
  Variant[Boolean,Enum['on_failure']]           $logoutput                = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}
  # TODO: This will only create a pluggable database on filesystem because of the
  #       pdb_datafile_destination which is in the 'creates' of the exec. We need
  #       to find a way to create the database in ASM too.

  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $ensure == present ) {
    unless ( $pdb_admin_username ) {fail('You must specify a pdb_admin_username') }
    unless ( $pdb_admin_password ) {fail('You must specify a pdb_admin_password') }
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  if ( $ensure == present ) {
    $command = "${oracle_home_dir}/bin/dbca -silent -createPluggableDatabase -sourceDB ${source_db} -pdbName ${pdb_name} -createPDBFrom DEFAULT -pdbAdminUserName ${pdb_admin_username} -pdbAdminPassword ${pdb_admin_password} -pdbDatafileDestination ${pdb_datafile_destination} -createUserTableSpace ${create_user_tablespace}"

    exec { "dbca pdb execute ${title}":
      command   => $command,
      timeout   => 0,
      path      => $exec_path,
      cwd       => $oracle_home_dir,
      user      => $user,
      group     => $group,
      creates   => $pdb_datafile_destination,
      logoutput => $logoutput,
    }
  } else {
    $command = "${oracle_home_dir}/bin/dbca -silent -deletePluggableDatabase -sourceDB ${source_db} -pdbName ${pdb_name}"

    exec { "dbca pdb execute ${title}":
      command   => $command,
      timeout   => 0,
      path      => $exec_path,
      cwd       => $oracle_home_dir,
      user      => $user,
      group     => $group,
      onlyif    => "ls ${pdb_datafile_destination}",
      logoutput => $logoutput,
    }

    -> file { $pdb_datafile_destination:
      ensure => absent,
      force  => true,
    }
  }
}
