#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::dbactions
#
#
# action        =  stop|start
#
#
define ora_install::dbactions(
  Enum['database','asm','grid']          $db_type     = 'database',
  Stdlib::Absolutepath                   $oracle_home = undef,
  Optional[Stdlib::Absolutepath]         $grid_home   = undef,
  String[1]                              $user        = 'oracle',
  String[1]                              $group       = 'dba',
  Enum['running','start','stop','abort'] $action      = 'start',
  String[1]                              $db_name     = 'orcl',
){
  ora_install { $title: ensure => 'present'}
  # TODO: Make this more logical. When specifying db_type = asm or grid it will
  #       stop ASM eventhough you specify a 'normal' database in db_name. I was
  #       thinking it would use srvctl to stop the database specified in db_name.
  #       It also does this from oracle_home, so why we have to specify the grid_home?

  if ( $db_type in ['grid','asm'] and !$grid_home ) {fail('You must specify grid_home if db_type is asm or grid')}

  if $db_type == 'database' {
    db_control{"instance control ${title}":
      ensure                  => $action,
      instance_name           => $db_name,
      oracle_product_home_dir => $oracle_home,
      os_user                 => $user,
    }
  } elsif $db_type in ['grid','asm'] {
    db_control{"instance control ${title}":
      ensure                  => $action,
      provider                => 'srvctl',
      instance_name           => $db_name,
      oracle_product_home_dir => $oracle_home,
      grid_product_home_dir   => $grid_home,
      os_user                 => $user,
      db_type                 => 'grid',
    }
  }
}
