#
# See the file "LICENSE" for the full license governing this code.
#
#
#
#
define ora_install::goldengate(
  Enum['12.1.2','12.2.1']             $version                   = '12.2.1',
  String[1]                           $file                      = undef,
  Enum['Oracle']                      $database_type             = 'Oracle',
  Enum['ORA11g','ORA12c']             $database_version          = 'ORA11g',
  Stdlib::Absolutepath                $database_home             = undef,
  Stdlib::Absolutepath                $oracle_base               = undef,
  Stdlib::Absolutepath                $goldengate_home           = undef,
  Integer                             $manager_port              = undef,
  String[1]                           $user                      = 'ggate',
  String[1]                           $group                     = 'dba',
  String[1]                           $group_install             = 'oinstall',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Stdlib::Absolutepath                $temp_dir                  = '/tmp',
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[Stdlib::Absolutepath]      $ora_inventory_dir         = undef,
)
{
  ora_install { $title: ensure => 'present'}

  $version_elements = split($version,'\.')
  $major_version = $version_elements[0]

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base}/oraInventory"
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  db_directory_structure{"oracle goldengate structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group_install,
  }

  unless ( oracle_exists( $goldengate_home ) ) {

    notify { "ora_install::goldengate ${goldengate_home} does not exists": }

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    $ggate_install_dir = 'fbo_ggs_Linux_x64_shiphome'

    archive { "${download_dir}/${file}":
      ensure       => present,
      cleanup      => $archive_cleanup,
      creates      => "${download_dir}/${ggate_install_dir}",
      extract      => true,
      extract_path => $download_dir,
      group        => $group,
      require      => Db_directory_structure["oracle goldengate structure ${version}"],
      source       => "${mount_point}/${file}",
      temp_dir     => $temp_dir,
      user         => $user,
    }

    file { "${download_dir}/oggcore.rsp":
      content => template("ora_install/oggcore_${major_version}.rsp.erb"),
      owner   => $user,
      group   => $group,
      require => Db_directory_structure["oracle goldengate structure ${version}"],
    }

    ora_install::utils::dborainst{"ggate orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
    }

    exec { 'install oracle goldengate':
      command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/${ggate_install_dir}/Disk1/runInstaller -silent -waitforcompletion -responseFile ${download_dir}/oggcore.rsp'",
      require     => [File["${download_dir}/oggcore.rsp"],
                      Ora_install::Utils::Dborainst["ggate orainst ${version}"],
                      Archive["${download_dir}/${file}"]],
      creates     => $goldengate_home,
      environment => ["TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      path        => $exec_path,
      logoutput   => $logoutput,
      user        => $user,
      group       => $group_install,
      returns     => [3,0],
    }

    # cleanup
    exec { "remove oracle goldengate extract folder ${download_dir}/${ggate_install_dir}":
      command   => "rm -rf ${download_dir}/${ggate_install_dir}",
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      require   => Exec['install oracle goldengate'],
      logoutput => $logoutput,
    }
    if ( $archive_cleanup ) {
      file { "${download_dir}/${file}":
        ensure => absent,
      }
    }
  }
}
