#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::installasm
#
#
define ora_install::installasm(
  # lint:ignore:2sp_soft_tabs
  Enum['12.2.0.1','12.1.0.1','12.1.0.2','11.2.0.4']
                                       $version                   = undef,
  String[1]                            $file                      = undef,
  Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']
                                       $grid_type                 = 'HA_CONFIG',
  Boolean                              $stand_alone               = true, # in case of 'CRS_SWONLY' and used as stand alone or in RAC
  Stdlib::Absolutepath                 $grid_base                 = undef,
  Stdlib::Absolutepath                 $grid_home                 = undef,
  Optional[Stdlib::Absolutepath]       $ora_inventory_dir         = undef,
  Boolean                              $bash_profile              = true,
  String[1]                            $user                      = 'grid',
  Stdlib::Absolutepath                 $user_base_dir             = '/home',
  String[1]                            $group                     = 'asmdba',
  String[1]                            $group_install             = 'oinstall',
  String[1]                            $group_oper                = 'asmoper',
  String[1]                            $group_asm                 = 'asmadmin',
  String[1]                            $sys_asm_password          = 'Welcome01',
  String[1]                            $asm_monitor_password      = 'Welcome01',
  String[1]                            $asm_diskgroup             = 'DATA',
  Optional[String[1]]                  $disk_discovery_string     = undef,
  Enum['NORMAL','HIGH','EXTERNAL']     $disk_redundancy           = 'NORMAL',
  Enum['1','2','4','8','16','32','64'] $disk_au_size              = '1',
  Optional[String[1]]                  $disks                     = undef,
  Optional[String[1]]                  $disks_failgroup_names     = undef,
  Optional[Boolean]                    $configure_afd             = false,
  Stdlib::Absolutepath                 $download_dir              = '/install',
  Stdlib::Absolutepath                 $temp_dir                  = '/tmp',
  Boolean                              $zip_extract               = true,
  Optional[String[1]]                  $puppet_download_mnt_point = undef,
  Optional[Boolean]                    $remote_file               = undef,
  Optional[String[1]]                  $cluster_name              = undef,
  Optional[String[1]]                  $scan_name                 = undef,
  Optional[Integer]                    $scan_port                 = undef,
  Optional[String[1]]                  $cluster_nodes             = undef,
  Optional[String[1]]                  $network_interface_list    = undef,
  Optional[Enum['FLEX_ASM_STORAGE','CLIENT_ASM_STORAGE','LOCAL_ASM_STORAGE','FILE_SYSTEM_STORAGE','ASM_STORAGE']]
                                       $storage_option            = undef,
  Variant[Boolean,Enum['on_failure']]  $logoutput                 = lookup({name => 'logoutput', default_value => 'on_failure'}),
  # lint:endignore
)
{
  ora_install { $title: ensure => 'present'}
  # TODO:
  #   - Check if this is still working for older versions
  #   - Check if this is still working for RAC
  #   - Check how to install ASM version 12.2 for RAC

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  $file_without_ext = regsubst($file, '(.+?)(\.zip*$|$)', '\1')

  if($cluster_name){ # We've got a RAC cluster. Check the cluster specific parameters
    unless ( $scan_name ) {fail('You must specify scan_name if cluster_name is defined') }
    unless ( $scan_port ) {fail('You must specify scan_port if cluster_name is defined') }
    unless ( $cluster_nodes ) {fail('You must specify cluster_nodes if cluster_name is defined') }
    unless ( $network_interface_list ) {fail('You must specify network_interface_list if cluster_name is defined') }
    unless ( $storage_option ) {fail('You must specify storage_option if cluster_name is defined') }
  }

  unless ( $::kernel in ['Linux','SunOS'] ){
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  }

  case $version {
    '11.2.0.4': {
      unless ( $storage_option in ['ASM_STORAGE','FILE_SYSTEM_STORAGE',undef] ) { fail("storage_option should be defined as ASM_STORAGE or FILE_SYSTEM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY'] ) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE or CRS_SWONLY for version ${version}") }
      $files = [$file]
      $extract_staged = true
      $ohas_service_template = 'ora_install/ohas.service.erb'
      $install_command = "${download_dir}/${file_without_ext}/grid/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = $grid_home
      $run_grid_user_config = false
    }
    '12.1.0.1', '12.1.0.2': {
      unless ( $storage_option in ['LOCAL_ASM_STORAGE','FLEX_ASM_STORAGE','FILE_SYSTEM_STORAGE',undef] ) { fail("storage_option should be defined as LOCAL_ASM_STORAGE, FLEX_ASM_STORAGE or FILE_SYSTEM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY'] ) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE, CRS_SWONLY or HA_SWONLY for version ${version}") }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $ohas_service_template = undef
      $install_command = "${download_dir}/${file_without_ext}/grid/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = $grid_home
      $run_grid_user_config = false
    }
    '12.2.0.1': {
      unless ( $storage_option in ['FLEX_ASM_STORAGE','CLIENT_ASM_STORAGE',undef] ) { fail("storage_option should be defined as FLEX_ASM_STORAGE or CLIENT_ASM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY'] ) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE, CRS_SWONLY or HA_SWONLY for version ${version}") }
      $files = ["${file}.zip"]
      $extract_staged = false
      $ohas_service_template = undef
      $install_command = "${grid_home}/gridSetup.sh -silent -waitForCompletion -skipPrereqs -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = "${grid_home}/install/config/oracle.rdbms.util.xml.ouibak"
      $run_grid_user_config = true
    }
    default: {}
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${grid_base}/oraInventory"
  }

  db_directory_structure{"grid structure ${version}":
    ensure            => present,
    oracle_base_dir   => $grid_base,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group_install,
  }

  unless ( oracle_exists( $grid_home ) ) {

    notify { "ora_install::installasm ${grid_home} does not exists": }

    $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    if ( $zip_extract ) {

      if ( $extract_staged ) {
        file { "${download_dir}/${file_without_ext}":
          ensure  => directory,
          owner   => $user,
          group   => $group,
          mode    => '0755',
          require => Db_directory_structure["grid structure ${version}"],
        }

        $files.each | $filename | {
          archive { "${download_dir}/${filename}":
            ensure       => present,
            before       => Exec["install oracle grid ${title}"],
            cleanup      => $archive_cleanup,
            creates      => "${download_dir}/${file}/installasm",
            extract      => true,
            extract_path => "${download_dir}/${file_without_ext}",
            group        => $group_install,
            require      => File["${download_dir}/${file_without_ext}"],
            source       => "${mount_point}/${filename}",
            temp_dir     => $temp_dir,
            user         => $user,
          }
        }
      } else {
        file { $grid_home:
          ensure  => directory,
          recurse => false,
          replace => false,
          mode    => '0775',
          owner   => $user,
          group   => $group_install,
          require => Db_directory_structure["grid structure ${version}"],
        }

        $files.each | $filename | {
          archive { "${download_dir}/${filename}":
            ensure       => present,
            before       => Exec["install oracle grid ${title}"],
            cleanup      => $archive_cleanup,
            creates      => "${grid_home}/installasm",
            extract      => true,
            extract_path => $grid_home,
            group        => $group_install,
            require      => File[$grid_home],
            source       => "${mount_point}/${filename}",
            temp_dir     => $temp_dir,
            user         => $user,
          }
        }
      }
    }

    ora_install::utils::dborainst{"grid orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
    }

    unless ( defined(File["${download_dir}/grid_install_${version}.rsp"]) ) {
      file { "${download_dir}/grid_install_${version}.rsp":
        ensure  => present,
        content => template("ora_install/grid_install_${version}.rsp.erb"),
        mode    => '0770',
        owner   => $user,
        group   => $group,
        require => [Ora_install::Utils::Dborainst["grid orainst ${version}"],
                    Db_directory_structure["grid structure ${version}"],],
      }
    }

    exec { "install oracle grid ${title}":
      command     => "/bin/sh -c 'unset DISPLAY;${install_command}'",
      creates     => $install_creates,
      environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      user        => $user,
      group       => $group_install,
      cwd         => $grid_base,
      logoutput   => $logoutput,
      require     => [Ora_install::Utils::Dborainst["grid orainst ${version}"],
                      File["${download_dir}/grid_install_${version}.rsp"]],
    }

    if ( $bash_profile ) {
      unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
        file { "${user_base_dir}/${user}/.bash_profile":
          ensure  => present,
          content => regsubst(template('ora_install/grid_bash_profile.erb'), '\r\n', "\n", 'EMG'),
          mode    => '0775',
          owner   => $user,
          group   => $group,
        }
      }
    }

    # because of RHEL7 uses systemd we need to create the service differently
    if ( $::osfamily == 'RedHat' ) and ( $::operatingsystemmajrelease == '7' ) {
      if ( $ohas_service_template ) {
        file {'/etc/systemd/system/ohas.service':
          ensure  => 'file',
          content => template($ohas_service_template),
          mode    => '0644',
          require => Exec["install oracle grid ${title}"],
        }

        -> exec { 'daemon-reload for ohas':
          command => '/bin/systemctl daemon-reload',
        }

        -> service { 'ohas.service':
          ensure => running,
          enable => true,
          before => Exec["run root.sh grid script ${title}"],
        }
      }
    }

    exec { "run root.sh grid script ${title}":
      timeout   => 0,
      command   => "${grid_home}/root.sh",
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      cwd       => $grid_base,
      logoutput => $logoutput,
      require   => Exec["install oracle grid ${title}"],
    }

    # cleanup
    if ( $zip_extract ) {
      exec { "remove oracle asm extract folder ${title}":
        command => "rm -rf ${download_dir}/${file_without_ext}",
        user    => 'root',
        group   => 'root',
        path    => $exec_path,
        require => Exec["install oracle grid ${title}"],
      }
      if ( $archive_cleanup ) {
        $files.each | $filename | {
          file { "${download_dir}/${filename}":
            ensure => absent,
          }
        }
      }
    }

    if ( $grid_type == 'CRS_SWONLY' ) {
      if ( $stand_alone ) {
        exec { 'Configuring Grid Infrastructure for a Stand-Alone Server':
          command   => "${grid_home}/perl/bin/perl -I${grid_home}/perl/lib -I${grid_home}/crs/install ${grid_home}/crs/install/roothas.pl",
          user      => 'root',
          group     => 'root',
          path      => $exec_path,
          cwd       => $grid_base,
          logoutput => $logoutput,
          require   => [Exec["run root.sh grid script ${title}"],
                        File[$grid_home],],
        }
      }
    } else {
      if ( $run_grid_user_config ) {
        exec { "run configTools grid script ${title}":
          timeout     => 0,
          command     => "/bin/sh -c 'unset DISPLAY;${grid_home}/gridSetup.sh -executeConfigTools -silent -skipPrereqs -responseFile ${download_dir}/grid_install_${version}.rsp'",
          user        => $user,
          group       => $group_install,
          environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
          path        => $exec_path,
          cwd         => $grid_base,
          logoutput   => $logoutput,
          returns     => [0,3], # when a scan adress is not defined in the DNS, it fails, but we can continue
          require     => Exec["run root.sh grid script ${title}"],
        }
      } else {
        file { "${download_dir}/cfgrsp.properties":
          ensure  => present,
          content => template('ora_install/grid_password.properties.erb'),
          mode    => '0600',
          owner   => $user,
          group   => $group,
          require => [Exec["run root.sh grid script ${title}"],],
        }

        exec { "run configToolAllCommands grid tool ${title}":
          timeout   => 0, # This can sometimes take a long time
          command   => "${grid_home}/cfgtoollogs/configToolAllCommands RESPONSE_FILE=${download_dir}/cfgrsp.properties",
          user      => $user,
          group     => $group_install,
          path      => $exec_path,
          provider  => 'shell',
          cwd       => "${grid_home}/cfgtoollogs",
          logoutput => $logoutput,
          returns   => [0,3], # when a scan adress is not defined in the DNS, it fails, but we can continue
          require   => [File["${download_dir}/cfgrsp.properties"],
                        Exec["run root.sh grid script ${title}"],
                        Exec["install oracle grid ${title}"],
                        ],
        }
      }
    }
  }
}
