#
# See the file "LICENSE" for the full license governing this code.
#
#
#
#
define ora_install::installem_agent(
  Enum['12.1.0.4','12.1.0.5','13.1.0.0','13.2.0.0']         $version                     = '12.1.0.5',
  Enum['agentPull','agentDeploy']                           $install_type                = undef,
  Enum['12.1.0.4.0','12.1.0.5.0','13.1.0.0.0','13.2.0.0.0'] $install_version             = '12.1.0.5.0',
  String[1]                                                 $install_platform            = 'Linux x86-64',
  Optional[String[1]]                                       $source                      = undef, # 'https://<OMS_HOST>:<OMS_PORT>/em/install/getAgentImage'|'/tmp/12.1.0.4.0_AgentCore_226_Linux_x64.zip'
  Optional[Stdlib::Absolutepath]                            $ora_inventory_dir           = undef,
  Stdlib::Absolutepath                                      $oracle_base_dir             = undef,
  Stdlib::Absolutepath                                      $agent_base_dir              = undef,
  Optional[Stdlib::Absolutepath]                            $agent_instance_home_dir     = undef,
  String[1]                                                 $agent_registration_password = undef,
  Integer                                                   $agent_port                  = 1830,
  String[1]                                                 $sysman_user                 = 'sysman',
  String[1]                                                 $sysman_password             = undef,
  String[1]                                                 $oms_host                    = undef,
  Integer                                                   $oms_port                    = undef,
  Integer                                                   $em_upload_port              = undef,
  String[1]                                                 $user                        = 'oracle',
  String[1]                                                 $group                       = 'oinstall',
  Stdlib::Absolutepath                                      $download_dir                = '/install',
  Optional[Variant[Boolean,Enum['on_failure']]]             $log_output                  = undef,
  Variant[Boolean, Enum['on_failure']]                      $logoutput                   = lookup({name => 'logoutput', default_value => 'on_failure'}),
)
{
  ora_install { $title: ensure => 'present'}

  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base_dir}/oraInventory"
  }

  if ( $version in ['12.1.0.4', '12.1.0.5'] ) {
    $agent_home = "${agent_base_dir}/core/${install_version}"
  }
  else {
    $agent_home = "${agent_base_dir}/agent_${install_version}"
  }

  # setup oracle base with the right permissions
  db_directory_structure{"oracle em agent structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base_dir,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group,
  }

  unless ( oracle_exists( $agent_home ) ) {

    notify { "ora_install::installem_agent ${agent_home} does not exists": }

    $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    # check oraInst
    ora_install::utils::dborainst{"em agent orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group,
    }

    if ( $source ) {
      $source_loc = $source
    } else {
      $source_loc = "https://${oms_host}:${em_upload_port}/em/install/getAgentImage"
    }

    if ( $source_loc == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    unless ( defined(Package['zip']) ) {
      package { 'zip':
        ensure  => present,
      }
    }

    # chmod +x /tmp/AgentPull.sh
    if ( $install_type  == 'agentPull') {

      if ( $sysman_password == undef or is_string($sysman_password) == false) {fail('You must specify sysman_password') }

      unless ( defined(Package['curl']) ) {
        package { 'curl':
          ensure  => present,
        }
      }

      exec { "agentPull ${title}":
        command   => "curl ${source_loc} --insecure -o ${download_dir}/AgentPull.sh",
        timeout   => 0,
        logoutput => $logoutput,
        path      => $exec_path,
        user      => $user,
        group     => $group,
        require   => [Package['curl'],
                      Db_directory_structure["oracle em agent structure ${version}"],],
      }

      exec { "chmod ${title}":
        command   => "chmod +x ${download_dir}/AgentPull.sh",
        timeout   => 0,
        logoutput => $logoutput,
        path      => $exec_path,
        user      => $user,
        group     => $group,
        require   => Exec["agentPull ${title}"],
      }

      file { "${download_dir}/em_agent.properties":
        ensure  => present,
        content => template('ora_install/em_agent_pull.properties.erb'),
        mode    => '0755',
        owner   => $user,
        group   => $group,
        require => Db_directory_structure["oracle em agent structure ${version}"],
      }

      $command = "${download_dir}/AgentPull.sh LOGIN_USER=${sysman_user} LOGIN_PASSWORD=${sysman_password} PLATFORM=\"${install_platform}\" VERSION=${install_version} AGENT_BASE_DIR=${agent_base_dir} AGENT_REGISTRATION_PASSWORD=${agent_registration_password} RSPFILE_LOC=${download_dir}/em_agent.properties"

      exec { "agentPull execute ${title}":
        command   => $command,
        timeout   => 0,
        logoutput => $logoutput,
        path      => $exec_path,
        user      => $user,
        group     => $group,
        require   => [Exec["agentPull ${title}"],
                      Exec["chmod ${title}"],
                      File["${download_dir}/em_agent.properties"],
                      Db_directory_structure["oracle em agent structure ${version}"],
                      Ora_install::Utils::Dborainst["em agent orainst ${version}"],],
      }

      exec { "run em agent root.sh script ${title}":
        command   => "${agent_home}/root.sh",
        user      => 'root',
        group     => 'root',
        path      => $exec_path,
        cwd       => $agent_base_dir,
        logoutput => $logoutput,
        require   => Exec["agentPull execute ${title}"],
      }

    } elsif ( $install_type  == 'agentDeploy') {

      unless ( $source ) {fail('You must specify source when using install_type => \'agentDeploy\'') }

      file { "${download_dir}/em_agent_${version}":
        ensure  => directory,
        owner   => $user,
        group   => $group,
        mode    => '0755',
        require => Db_directory_structure["oracle em agent structure ${version}"],
      }

      $basename_source = split($source_loc, '/')[-1]

      archive { "${download_dir}/${basename_source}":
        ensure         => present,
        allow_insecure => true,
        before         => Exec["agentDeploy execute ${title}"],
        cleanup        => $archive_cleanup,
        extract        => true,
        extract_path   => "${download_dir}/em_agent_${version}",
        group          => $group,
        require        => File["${download_dir}/em_agent_${version}"],
        source         => $source_loc,
        user           => $user,
      }

      if ( $agent_instance_home_dir ) {
        $command = "${download_dir}/em_agent_${version}/agentDeploy.sh AGENT_BASE_DIR=${agent_base_dir} AGENT_INSTANCE_HOME=${agent_instance_home_dir} AGENT_REGISTRATION_PASSWORD=${agent_registration_password} OMS_HOST=${oms_host} AGENT_PORT=${agent_port} EM_UPLOAD_PORT=${em_upload_port}"
      } else {
        $command = "${download_dir}/em_agent_${version}/agentDeploy.sh AGENT_BASE_DIR=${agent_base_dir} AGENT_REGISTRATION_PASSWORD=${agent_registration_password} OMS_HOST=${oms_host} AGENT_PORT=${agent_port} EM_UPLOAD_PORT=${em_upload_port}"
      }

      exec { "agentDeploy execute ${title}":
        command   => $command,
        cwd       => '/tmp',
        timeout   => 0,
        logoutput => $logoutput,
        path      => $exec_path,
        user      => $user,
        group     => $group,
        require   => [Archive["${download_dir}/${basename_source}"],
                      Db_directory_structure["oracle em agent structure ${version}"],
                      Ora_install::Utils::Dborainst["em agent orainst ${version}"],],
      }

      exec { "run em agent root.sh script ${title}":
        command   => "${agent_home}/root.sh",
        user      => 'root',
        group     => 'root',
        path      => $exec_path,
        cwd       => $agent_base_dir,
        logoutput => $logoutput,
        require   => Exec["agentDeploy execute ${title}"],
      }
    }
  }
}
