#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::net
#
define ora_install::net(
  Stdlib::Absolutepath                $oracle_home  = undef,
  Enum['11.2','12.1','12.2']          $version      = '11.2',
  String[1]                           $user         = 'oracle',
  String[1]                           $group        = 'dba',
  Integer                             $db_port      = 1521,
  Stdlib::Absolutepath                $download_dir = '/install',
  Stdlib::Absolutepath                $temp_dir     = '/tmp',
  Variant[Boolean,Enum['on_failure']] $logoutput    = lookup({name => 'logoutput', default_value => 'on_failure'}),
){
  ora_install { $title: ensure => 'present'}

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  file { "${download_dir}/netca_${version}.rsp":
    ensure  => present,
    content => template("ora_install/netca_${version}.rsp.erb"),
    mode    => '0775',
    owner   => $user,
    group   => $group,
  }

  exec { "install oracle net ${title}":
    command     => "${oracle_home}/bin/netca -silent -responsefile ${download_dir}/netca_${version}.rsp",
    cwd         => $temp_dir,
    require     => File["${download_dir}/netca_${version}.rsp"],
    creates     => "${oracle_home}/network/admin/listener.ora",
    path        => $exec_path,
    user        => $user,
    group       => $group,
    environment => ["USER=${user}",],
    logoutput   => $logoutput,
  }
}
