#
# See the file "LICENSE" for the full license governing this code.
#
# == Define: ora_install::opatch
#
# installs oracle patches for Oracle products
#
#
define ora_install::opatch(
  Enum[present, absent] $ensure                    = present,
  Stdlib::Absolutepath  $oracle_product_home       = undef,
  Integer               $patch_id                  = undef,
  String[1]             $patch_file                = undef,
  Boolean               $clusterware               = false, # opatch auto or opatch apply
  Optional[Integer]     $bundle_sub_patch_id       = undef,
  Optional[Integer]     $bundle_sub_folder         = undef,
  String[1]             $user                      = 'oracle',
  String[1]             $group                     = 'dba',
  Stdlib::Absolutepath  $download_dir              = '/install',
  Boolean               $ocmrf                     = false,
  Optional[String[1]]   $puppet_download_mnt_point = undef,
  Optional[Boolean]     $remote_file               = undef,
)
{
  ora_install { $title: ensure => 'present'}

  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  case $::kernel {
    'Linux': {
      $ora_inst_path = '/etc'
    }
    'SunOS': {
      $ora_inst_path = '/var/opt/oracle'
    }
    default: {
        fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }
  }

  if ( $puppet_download_mnt_point ) {
    $mount_point =  $puppet_download_mnt_point
  } else {
    $mount_point = 'puppet:///modules/ora_install/'
  }

  case $::kernel {
    'Linux', 'SunOS': {
      if $ensure == 'present' {
        archive { "${download_dir}/${patch_file}":
          ensure       => present,
          user         => $user,
          group        => $group,
          extract      => true,
          extract_path => $download_dir,
          source       => "${mount_point}/${patch_file}",
          creates      => "${download_dir}/${patch_id}",
          cleanup      => false,
        }
      }

      # sometimes the bundle patch inside an other folder
      if ( $bundle_sub_folder ) {
        $extracted_patch_dir = "${download_dir}/${patch_id}/${bundle_sub_folder}"
      } else {
        $extracted_patch_dir = "${download_dir}/${patch_id}"
      }

      if ( $ocmrf ) {

        db_opatch{ "${patch_id} ${title}":
          ensure                  => $ensure,
          patch_id                => $patch_id,
          os_user                 => $user,
          oracle_product_home_dir => $oracle_product_home,
          orainst_dir             => $ora_inst_path,
          extracted_patch_dir     => $extracted_patch_dir,
          ocmrf_file              => "${oracle_product_home}/OPatch/ocm.rsp",
          bundle_sub_patch_id     => $bundle_sub_patch_id,
          opatch_auto             => $clusterware,
        }

      } else {

        db_opatch{ "${patch_id} ${title}":
          ensure                  => $ensure,
          patch_id                => $patch_id,
          os_user                 => $user,
          oracle_product_home_dir => $oracle_product_home,
          orainst_dir             => $ora_inst_path,
          extracted_patch_dir     => $extracted_patch_dir,
          bundle_sub_patch_id     => $bundle_sub_patch_id,
          opatch_auto             => $clusterware,
        }

      }

    }
    default: {
      fail('Unrecognized operating system')
    }
  }
}
