#
# See the file "LICENSE" for the full license governing this code.
#
# == Class: ora_install::tnsnames
#
#
define ora_install::tnsnames(
  Stdlib::Absolutepath        $oracle_home          = undef,
  String[1]                   $user                 = 'oracle',
  String[1]                   $group                = 'dba',
  Hash                        $server               = {myserver => { host => undef, port => '1521', protocol => 'TCP' }},
  String[1]                   $loadbalance          = 'ON',
  String[1]                   $failover             = 'ON',
  String[1]                   $connect_service_name = undef,
  String[1]                   $connect_server       = 'DEDICATED',
  Enum['tnsname','listener']  $entry_type           = 'tnsname',
)
{
  ora_install { $title: ensure => 'present'}

  unless ( defined(Concat["${oracle_home}/network/admin/tnsnames.ora"]) ) {
    concat { "${oracle_home}/network/admin/tnsnames.ora":
      ensure         => present,
      owner          => $user,
      group          => $group,
      mode           => '0774',
      ensure_newline => true,
    }
  }

  case $entry_type {
    'tnsname'  : { $template_path = 'ora_install/tnsnames.erb' }
    'listener' : { $template_path = 'ora_install/listener.erb' }
    default    : { fail("${entry_type} is not a supported entry_type") }
  }

  concat::fragment { $title:
    target  => "${oracle_home}/network/admin/tnsnames.ora",
    content => template($template_path),
  }
}
